import asyncio
import secrets
from typing import Any, Literal, Sequence, cast

from lueur.make_id import make_id
from lueur.models import Discovery, Meta
from lueur.platform.grafana.slo import expand_links as slo_expand_links
from lueur.platform.grafana.slo import explore_slo

__all__ = ["explore", "expand_links"]

Targets = ("slo",)


async def explore(
    stack_url: str,
    token: str,
    include: Sequence[Literal["slo",]] | None = None,
) -> Discovery:
    resources = []
    tasks: list[asyncio.Task] = []

    if include is None:
        include = cast(Sequence, Targets)

    async with asyncio.TaskGroup() as tg:
        if "slo" in include:
            tasks.append(tg.create_task(explore_slo(stack_url, token)))

    for task in tasks:
        result = task.result()
        if result is None:
            continue
        resources.extend(result)

    name = secrets.token_hex(8)

    return Discovery(
        id=make_id(name),
        resources=resources,
        meta=Meta(name=name, display="Grafana", kind="grafana", category=None),
    )


def expand_links(d: Discovery, serialized: dict[str, Any]) -> None:
    slo_expand_links(d, serialized)



# mypy: disable-error-code="index,call-arg,union-attr"
import logging
from typing import Any

import httpx
import msgspec

from lueur.make_id import make_id
from lueur.models import Discovery, K8SMeta, Resource
from lueur.platform.grafana.client import Client

__all__ = ["explore_slo", "expand_links"]
logger = logging.getLogger("lueur.lib")


async def explore_slo(
    stack_url: str,
    token: str,
) -> list[Resource]:
    resources = []

    async with Client(stack_url, token) as c:
        nodes = await explore_slos(c)
        resources.extend(nodes)

    return resources


###############################################################################
# Private functions
###############################################################################
async def explore_slos(c: httpx.AsyncClient) -> list[Resource]:
    response = await c.get("/api/plugins/grafana-slo-app/resources/v1/slo")

    if response.status_code == 403:
        logger.warning("Grafana API authentication failed")
        return []

    slos = msgspec.json.decode(response.content)

    results = []
    for slo in slos["items"]:
        results.append(
            Resource(
                id=make_id(slo["id"]),
                meta=K8SMeta(
                    name=slo["name"],
                    display=slo["name"],
                    kind="slo",
                    platform="grafana",
                    category="observability",
                ),
                struct=slo,
            )
        )

    return results


def expand_links(d: Discovery, serialized: dict[str, Any]) -> None:
    pass



from contextlib import asynccontextmanager
from typing import AsyncGenerator

import httpx

__all__ = ["Client"]


@asynccontextmanager
async def Client(
    stack_url: str, token: str
) -> AsyncGenerator[httpx.AsyncClient, None]:
    headers = {"Authorization": f"Bearer ${token}"}

    async with httpx.AsyncClient(
        http2=True, base_url=stack_url, headers=headers
    ) as c:
        yield c
